require_dependency "skirental_frontend_api/application_controller"

module SkirentalFrontendApi
    class BookingsController < ApplicationController

        def context
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            render json: booking
        end

        def create
            # check whether booking was already created. This should not happen, but..
            new_booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if new_booking.nil?
                new_booking = Booking.new(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id], context: booking_params[:booking].to_h)
            else
                # TODO: should be a fail!
                new_booking.context = booking_params[:booking].to_h
            end
            if new_booking.save
                render json: { error: "" }, :status => :created
            else
                render json: { error: new_booking.errors.full_messages }, :status => :unprocessable_entity
            end

        end

        def update
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first

            if booking.nil?
                render json: { error: "not found" }, :status => :not_found
            else
                already_finished = false
                unless booking.payments.blank?
                    booking.payments.each do |payment|
                        if payment.is_payed or payment.is_reserved
                            already_finished = true
                            break
                        end
                    end
                end
                unless already_finished
                    booking.context = booking_params[:booking].to_h
                    if booking.save
                        render json: { error: "", booking_id: booking.booking_id, voucher: booking.voucher }, :status => :ok
                    else
                        render json: { error: booking.errors.full_messages }, :status => :unprocessable_entity
                    end
                else
                    # don't allow everything -> only customer.personalData and product.selectedVariant can be updated from now on…
                    begin
                        # TODO: Update for new structure
                        updated_context = booking_params[:booking].to_h

                        # update booking.code once if not set yet (directly after payment)
                        if booking.context["code"].blank? and updated_context["code"].present?
                            booking.context["code"] = updated_context["code"]
                        end

        
                        # Iterate over updated customers
                        updated_context["customers"].each_with_index do |customer, customer_index|
                            # Overwrite personalData in persisted context
                            booking.context["customers"][customer_index]["personalData"] = customer["personalData"]

                            # Overwrite selectedVariants in persisted context
                            customer["packages"].each_with_index do |package, package_index|
                                booking.context["customers"][customer_index]["packages"][package_index]["mainProduct"]["selectedVariant"] = package["mainProduct"]["selectedVariant"]
                                package["accessories"].each_with_index do |accessorie, accessorie_index|
                                    booking.context["customers"][customer_index]["packages"][package_index]["accessories"][accessorie_index]["selectedVariant"] = accessorie["selectedVariant"]
                                end
                            end
                            customer["products"].each_with_index do |product, product_index|
                                booking.context["customers"][customer_index]["products"][product_index]["selectedVariant"] = product["selectedVariant"]
                            end
                        end

                        if booking_params[:no_sync].to_s.present?
                            if booking_params[:no_sync].to_s == 'true'
                                booking.no_sync = true
                            end
                        end

                        if booking.save
                            render json: { error: "", booking_id: booking.booking_id, voucher: booking.voucher }, :status => :ok
                        else
                            render json: { error: booking.errors.full_messages }, :status => :unprocessable_entity
                        end
                    rescue Exception => e
                        # most probably the lengths of customers and/or products arrays didn't match…
                        render json: { error: "already paid or reserved: #{e}" }, :status => :unprocessable_entity
                    end
                end
            end
        end

        private

        def booking_params
            params.permit(:shop_id, :session_id, :no_sync, booking: {})
        end
    end
end
