class Float
    def round_to(i)
        f = (10 ** i).to_f
        nr = self * f
        return nr.round / f
    end
end

module SkirentalFrontendApi
    class Booking
        include NoBrainer::Document
        include NoBrainer::Document::Timestamps

        before_save :sync_with_backend

        has_many :payments, class_name:SkirentalPayment::Payment, dependent: :destroy

        field  :session_id,             :type => String,                                    :required => true,   :index => true
        field  :shop_id,                :type => String,                                    :required => true,   :index => true
        field  :context,                :type => Hash,     :default => {}
        field  :booking_id,             :type => String
        field  :no_sync,                :type => Boolean,   :default => false

        def voucher
            # TODO: Rethink whether this is good for the future. Future: If we want to update bookings or need
            # receipts, we need to store the booking transaction into the payment and have to handle multiple (old) payments
            confirmed_payment = self.payments.order_by(:created_at => :desc).select { |payment|  payment.voucher_code != nil }.last
            if confirmed_payment.present?
                confirmed_payment.voucher_code
            else
                nil
            end
        end

        def last_payment
            self.payments.order_by(:created_at => :desc).last
        end

        def transaction(tax_include = true, tax_show = true)
            current_locale = context[:locale] || "de"
            current_locale = current_locale.split("-").first

            shop_settings = SkirentalApi::Client.settings(shop_id, current_locale)
            fail if context.nil?

            items = []
            start_date = Date.parse(context[:startDate]).strftime("%Y-%m-%d")
            duration = (Date.parse(context[:endDate]) - Date.parse(context[:startDate])).to_i + 1
            currency = shop_settings[:money][:payment][:cur]
            price_total = 0
            tax_total = 0

            context[:customers].each do |customer|
                age = customer[:customerGroup][:ageFrom]
                use_insurance = customer[:useInsurance]

                customer[:packages].each do |package|
                    # First calculate the main product
                    product_name = package[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : "")
                    description  = package[:description]
                    item = fetch_price_item(product_name, description, package[:mainProduct][:id], start_date, duration, age, shop_id, package[:id], currency, tax_include, tax_show, use_insurance, current_locale, package[:mainProduct][:count])
                    unless item.nil?
                        items.push item
                    end

                    # Add Accessories
                    package[:accessories].each do |accessory|
                        product_name = accessory[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : "")
                        description  = accessory[:description]
                        accessory_item = fetch_price_item(product_name, description, accessory[:id], start_date, duration, age, shop_id, package[:id], currency, tax_include, tax_show, use_insurance, current_locale, accessory[:count])
                        next if accessory_item.nil?
                        items.push accessory_item
                    end
                end

                customer[:products].each do |product|
                    product_name = product[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : "")
                    description  = product[:description]
                    product_item = fetch_price_item(product_name, description, product[:id], start_date, duration, age, shop_id, nil, currency, tax_include, tax_show, use_insurance, current_locale, product[:count])
                    next if product_item.nil?
                    items.push product_item
                end

                self.save
            end

            items.each do |item|
                price_total += item[:price].to_d * item[:quantity]
                if tax_show
                    tax_total += item[:tax].to_d * item[:quantity]
                end
            end


            transaction = {
                :item_list => {
                    :items => items.map do |item|
                        item[:price] = '%.2f' % item[:price].to_f.round(2)
                        item
                    end
                },
                :amount =>  {
                    :total    => '%.2f' % ((price_total + tax_total).to_f.round_to(2)),
                    :currency => currency,
                    :details  => !tax_show ? nil : {
                        :subtotal => '%.2f' % price_total.to_f.round_to(2),
                        :tax      => '%.2f' % tax_total.to_f.round_to(2)
                    }
                },
                :description => "Wir bedanken uns fuer Ihre Buchung!" # TODO: do we need a description???
            }

            transaction

        end

        def sync_with_backend
            if self.no_sync
                self.no_sync = false
            else
                api_booking_id = SkirentalApi::Client.sync_booking_state(self)
                self.booking_id = api_booking_id if api_booking_id.present?
            end
            true
        end

        def fetch_price_item(name, description, product_id, start_date, duration, age, shop_id, package_id, currency, tax_include, tax_show, use_insurance, current_locale, quantity)
            # 1. We expect that prices are independent to the lang.
            # 2. We store product_id's therefore we do not access prices of packages.
            price = SkirentalApi::Client.get_price_of_item(product_id, start_date, duration, age, shop_id, current_locale, package_id)
            fail if price.nil?
            return nil if price[:price].blank? || price[:price] <= 0 || price[:priceWithInsurance].blank? || price[:priceWithInsurance] <= 0

            price_value = use_insurance ? price[:priceWithInsurance] : price[:price]
            tax_value = use_insurance ? price[:vatValueWithInsurance] : price[:vatValue]

            if tax_include and tax_show
                price_value = (price_value.to_d - tax_value.to_d).to_f #paypal wants net prices!
            end

            {
                :name        => name,
                :description => description,
                :quantity    => quantity,
                :price       => price_value,
                :tax         => tax_show ? tax_value : nil,
                :sku         => product_id,
                :currency    => currency
            }
        end
    end
end
